#!/usr/bin/ruby
require_relative 'player'
require_relative 'GameTurn'
require_relative 'treasure_trove'

module StudioGame
    class Game
        attr_reader :name, :players
        def initialize name
            @name = name
            @players = []
            @number_players = 0
            @center = 40
            @alignment = 38
        end
        def add_player player
            @players.push(player)
            @number_players += 1
        end

        def print_game_intro
            puts "-" * @center
            puts "|" + "There are #{@number_players} players in #{@name}".center(@alignment) + "|"
            puts "-" * @center
        end
        def print_game_number num
            puts "*" * @center
            puts "*" + "Playing Game #{num}.".center(@alignment) + "*"
            puts "*" * @center
        end
        def play num_games
            print_game_intro
            print_all_treasures
            1.upto(num_games) do |num|
                print_game_number(num)
                @players.each do |player|
                    puts " #{player.name}'s Turn ".center(@center, "-")
                    player.found_treasure(find_random_treasures)
                    GameTurn.take_turn(player)
                    puts
                end
            end
            print_high_scores        
        end

        def find_random_treasures
            treasures = TreasureTrove::TREASURES
            treasures[rand(treasures.length)]        
        end
        def print_all_treasures
            treasures = TreasureTrove::TREASURES
            puts "-" * @center
            puts "|" + "There are #{treasures.length} treasures in this game".center(@alignment) + "|"
            puts "-" * @center
            treasures.each do |t|
                puts "|" + "#{t.name} is worth #{t.points} points.".center(@alignment) + "|"
            end
            puts "-" * @center
        end
        def print_high_scores
            puts "\nPrinting High Scores:"
            high_scores = @players.sort{|a, b| b.score <=> a.score}
            high_scores.each do |player| 
                puts "#{player.name}".ljust(@alignment, ".") + "#{player.score}"
            end
            puts "\nTotal Points accumulated #{total_points} from Treasure Trove\n\n"
        end

        def print_weak_strong
            strong, weak = @players.partition{|n| n.strong?}
            puts "Top #{strong.length} STRONG players."
            puts strong.sort
            puts "\nTop #{weak.length} WEAK players."
            puts weak.sort
            puts
        end 

        def print_player_stats
            @players.sort.each do |player|
                puts " Player #{player.name}'s point breakdown ".center(@center, "*")
                puts "-" * @center
                player.each_found_treasure do |treasure|
                    puts "|" + "#{treasure.name} #{treasure.points} points".center(@alignment) + "|"
                end
                puts "-" * @center
                puts "#{player.name}'s total points: #{player.points}"
                puts "#{player.name}'s total health: #{player.health}"
                puts
            end
        end
        def print_stats_header
            puts "-" * @center
            puts "|" + "#{@name} Statistics".center(@alignment) + "|"
            puts "-" * @center
        end
        def print_stats
            print_stats_header
            print_weak_strong
            print_player_stats
        end
        def total_points
            @players.reduce(0) { |sum, player| sum + player.points }
        end

        def load_players array
            array.each do |file|
                File.readlines(file).each do |line|
                    name, health = line.split(",")
                    add_player(Player.new(name, Integer(health)))
                end
            end
        end
        def save_high_scores file_name="high_scores.txt"
            File.open(file_name, "w") do |fd|
                fd.puts "#{@name} High Scores:"
                @players.sort.each do |player|
                    fd.puts "#{player.name},#{player.score}"
                end
            end
        end
    end
end

if __FILE__ == $0
    @game = Game.new("Knuckleheads")
    @initial_health = 100
    @player = Player.new("jeff", @initial_health)
    @player2 = Player.new("nick", @initial_health)
    @game.add_player(@player)
    @game.add_player(@player2)
    @game.play(2)
    @game.print_stats
end